/*
 * Copyright 2009 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.isam.dummy;

import java.util.logging.Logger;

import dk.vajhoej.isam.IsamException;
import dk.vajhoej.isam.IsamResult;
import dk.vajhoej.record.RecordException;

/**
 * Class DummyIsamResult is a "do nothing" implementation of ISAM result.
 * All writes succeed. All reads return nothing. 
 * It is only intended for testing purposes.
 */
public class DummyIsamResult<T> implements IsamResult<T> {
    public static Logger log = Logger.getLogger(DummyIsamResult.class.getName());
    public boolean read() throws IsamException, RecordException {
        log.warning("DummyIsamResult read called");
        return false;
    }
    public T current() throws IsamException, RecordException {
        log.warning("DummyIsamResult current called");
        return null;
    }
    public void close() throws IsamException {
        log.warning("DummyIsamResult close called");
    }
}
