/*
 * Copyright 2009 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.isam.test;

import static org.junit.Assert.*;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.Statement;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import dk.vajhoej.isam.IsamException;
import dk.vajhoej.isam.IsamSource;
import dk.vajhoej.isam.Key;
import dk.vajhoej.isam.KeyInfoArrayCache;
import dk.vajhoej.isam.cache.CacheIsamSource;
import dk.vajhoej.isam.dummy.DummyIsamSource;
import dk.vajhoej.isam.emulated.EmulatedIsamSource;
import dk.vajhoej.record.RecordException;

public class TestCacheIsamSource {
	private final static String DRIVER = "com.mysql.jdbc.Driver";
	private final static String CONURL = "jdbc:mysql://localhost/Test";
	private final static String UN = "";
	private final static String PW = "";
	@Before
	public void setup() {
		try {
			Class.forName(DRIVER);
			Connection con = DriverManager.getConnection(CONURL, UN, PW);
			Statement stmt = con.createStatement();
			stmt.executeUpdate("CREATE TABLE data (iv INTEGER NOT NULL, recblob BLOB, PRIMARY KEY(iv))");
			stmt.close();
			con.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	@After
	public void teardown() {
		try {
			Connection con = DriverManager.getConnection(CONURL, UN, PW);
			Statement stmt = con.createStatement();
			stmt.executeUpdate("DROP TABLE data");
			stmt.close();
			con.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
    @Test
    public void testCreateRead() {
        try {
            TestCommonIsamSource.testCreateRead(new CacheIsamSource(new EmulatedIsamSource(DRIVER, CONURL, UN, PW)));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testCreateReadKey0() {
        try {
            TestCommonIsamSource.testCreateReadKey0(new CacheIsamSource(new EmulatedIsamSource(DRIVER, CONURL, UN, PW)));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
	@Test
	public void testCreateReadMulti() {
		try {
			TestCommonIsamSource.testCreateReadMulti(new CacheIsamSource(new EmulatedIsamSource(DRIVER, CONURL, UN, PW)));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testReadPerf() {
		try {
			TestCommonIsamSource.testReadPerf(new CacheIsamSource(new EmulatedIsamSource(DRIVER, CONURL, UN, PW)));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testCreatePerf() {
		try {
			TestCommonIsamSource.testCreatePerf(new CacheIsamSource(new EmulatedIsamSource(DRIVER, CONURL, UN, PW)));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testReadGE() {
		try {
			TestCommonIsamSource.testReadGE(new CacheIsamSource(new EmulatedIsamSource(DRIVER, CONURL, UN, PW)));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testReadGT() {
		try {
			TestCommonIsamSource.testReadGT(new CacheIsamSource(new EmulatedIsamSource(DRIVER, CONURL, UN, PW)));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testReadLE() {
		try {
			TestCommonIsamSource.testReadLE(new CacheIsamSource(new EmulatedIsamSource(DRIVER, CONURL, UN, PW)));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testReadLT() {
		try {
			TestCommonIsamSource.testReadLT(new CacheIsamSource(new EmulatedIsamSource(DRIVER, CONURL, UN, PW)));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testError1() {
		try {
			TestCommonIsamSource.testError1(new CacheIsamSource(new EmulatedIsamSource(DRIVER, CONURL, UN, PW)));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
    @Test
    public void testError2() {
        try {
            TestCommonIsamSource.testError2(new CacheIsamSource(new EmulatedIsamSource(DRIVER, CONURL, UN, PW)));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testUpdate() {
        try {
            TestCommonIsamSource.testUpdate(new CacheIsamSource(new EmulatedIsamSource(DRIVER, CONURL, UN, PW)));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testDelete() {
        try {
            TestCommonIsamSource.testDelete(new CacheIsamSource(new EmulatedIsamSource(DRIVER, CONURL, UN, PW)));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testLogging() {
        try {
            TestCommonIsamSource.testLogging(new CacheIsamSource(new EmulatedIsamSource(DRIVER, CONURL, UN, PW)));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testConcurrency() {
        try {
            TestCommonIsamSource.testConcurrency(new CacheIsamSource(new EmulatedIsamSource(DRIVER, CONURL, UN, PW)),
                                                 new CacheIsamSource(new EmulatedIsamSource(DRIVER, CONURL, UN, PW)));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testConcurrencyHard() {
        try {
            TestCommonIsamSource.testConcurrencyHard(new CacheIsamSource(new EmulatedIsamSource(DRIVER, CONURL, UN, PW)),
                                                     new CacheIsamSource(new EmulatedIsamSource(DRIVER, CONURL, UN, PW)));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
	@Test
	public void testAnalyzeCache() {
		try {
			IsamSource isam = new CacheIsamSource(new EmulatedIsamSource(DRIVER, CONURL, UN, PW));
			KeyInfoArrayCache.getInstance().reset();
			Data o = new Data();
			o.setIv(258);
			o.setXv(123.456);
			o.setSv("ABC     ");
			isam.create(o);
			assertTrue("hit rate", KeyInfoArrayCache.getInstance().getHitRate() < 0.501);
			for(int i = 1; i < 10; i++) {
				Data o2 = new Data();
				o2.setIv(258 + i);
				o2.setXv(123.456);
				o2.setSv("ABC     ");
				isam.create(o2);
			}
			assertTrue("hit rate", KeyInfoArrayCache.getInstance().getHitRate() > 0.949);
			for(int i = 0; i < 10; i++) {
				Data o2 = isam.read(Data.class, new Key<Integer>(0, 258 + i));
				assertEquals("iv", 258 + i, o2.getIv());
			}
			assertTrue("hit rate", KeyInfoArrayCache.getInstance().getHitRate() > 0.949);
			isam.close();
		} catch (IsamException e) {
			e.printStackTrace();
			fail("Unexpected exception: " + e);
		} catch (RecordException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testDataCache() {
		try {
			CacheIsamSource isam = new CacheIsamSource(new EmulatedIsamSource(DRIVER, CONURL, UN, PW));
			assertTrue("hit rate", isam.getHitRate() < 0.001);
			Data o = new Data();
			o.setIv(258);
			o.setXv(123.456);
			o.setSv("ABC     ");
			isam.create(o);
			assertTrue("hit rate", isam.getHitRate() < 0.001);
			for(int i = 1; i < 10; i++) {
				Data o2 = new Data();
				o2.setIv(258 + i);
				o2.setXv(123.456);
				o2.setSv("ABC     ");
				isam.create(o2);
			}
			assertTrue("hit rate", isam.getHitRate() < 0.001);
			for(int i = 0; i < 10; i++) {
				Data o2 = isam.read(Data.class, new Key<Integer>(0, 258 + i));
				assertEquals("iv", 258 + i, o2.getIv());
			}
			assertTrue("hit rate", isam.getHitRate() > 0.999);
			isam.close();
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		} catch (RecordException e) {
			fail("Unexpected exception: " + e);
		}
	}
    @Test
    public void testDataCacheAnomaly() {
        try {
            CacheIsamSource isam = new CacheIsamSource(new DummyIsamSource());
            MultiData o;
            o = isam.read(MultiData.class, new Key<Integer>(0, 1));
            assertNull("before create with key 0", o);
            o = isam.read(MultiData.class, new Key<Integer>(1, 101));
            assertNull("before create with key 1", o);
            MultiData o2 = new MultiData();
            o2.setIv1(1);
            o2.setIv2(101);
            o2.setSv("ABC");
            isam.create(o2);
            o = isam.read(MultiData.class, new Key<Integer>(0, 1));
            assertNotNull("after create with key 0", o);
            assertEquals("sv after create", "ABC", o.getSv());
            o = isam.read(MultiData.class, new Key<Integer>(1, 101));
            assertNotNull("after create with key 1", o);
            assertEquals("sv after create", "ABC", o.getSv());
            o2.setSv("DEF");
            isam.update(o2);
            o = isam.read(MultiData.class, new Key<Integer>(0, 1));
            assertNotNull("after update with key 0", o);
            assertEquals("sv after update", "DEF", o.getSv());
            o = isam.read(MultiData.class, new Key<Integer>(1, 101));
            assertNotNull("after update with key 1", o);
            assertEquals("sv after update", "DEF", o.getSv());
            isam.delete(MultiData.class, new Key<Integer>(0, 1));
            o = isam.read(MultiData.class, new Key<Integer>(0, 1));
            assertNull("after delete with key 0", o);
            o = isam.read(MultiData.class, new Key<Integer>(1, 101));
            assertNull("after delete with key 1", o);
            isam.close();
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        } catch (RecordException e) {
            fail("Unexpected exception: " + e);
        }
    }
}
