/*
 * Copyright 2009 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.isam.test;

import static org.junit.Assert.*;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.Statement;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import dk.vajhoej.isam.IsamException;
import dk.vajhoej.isam.remote.RemoteIsamSource;

public class TestRemoteIsamSource {
    private final static String DRIVER = "com.mysql.jdbc.Driver";
    private final static String CONURL = "jdbc:mysql://localhost/Test";
    private final static String UN = "";
    private final static String PW = "";
    @Before
    public void setup() {
        try {
            Class.forName(DRIVER);
            Connection con = DriverManager.getConnection(CONURL, UN, PW);
            Statement stmt = con.createStatement();
            stmt.executeUpdate("CREATE TABLE data (iv INTEGER NOT NULL, recblob BLOB, PRIMARY KEY(iv))");
            stmt.close();
            con.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    @After
    public void teardown() {
        try {
            Connection con = DriverManager.getConnection(CONURL, UN, PW);
            Statement stmt = con.createStatement();
            stmt.executeUpdate("DROP TABLE data");
            stmt.close();
            con.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    @Test
    public void testCreateRead() {
        try {
            TestCommonIsamSource.testCreateRead(new RemoteIsamSource("localhost", 1234));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testCreateReadKey0() {
        try {
            TestCommonIsamSource.testCreateReadKey0(new RemoteIsamSource("localhost", 1234));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
	@Test
	public void testCreateReadMulti() {
		try {
			TestCommonIsamSource.testCreateReadMulti(new RemoteIsamSource("localhost", 1234));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testReadPerf() {
		try {
			TestCommonIsamSource.testReadPerf(new RemoteIsamSource("localhost", 1234));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testCreatePerf() {
		try {
			TestCommonIsamSource.testCreatePerf(new RemoteIsamSource("localhost", 1234));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testReadGE() {
		try {
			TestCommonIsamSource.testReadGE(new RemoteIsamSource("localhost", 1234));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testReadGT() {
		try {
			TestCommonIsamSource.testReadGT(new RemoteIsamSource("localhost", 1234));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testReadLE() {
		try {
			TestCommonIsamSource.testReadLE(new RemoteIsamSource("localhost", 1234));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testReadLT() {
		try {
			TestCommonIsamSource.testReadLT(new RemoteIsamSource("localhost", 1234));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testError1() {
		try {
			TestCommonIsamSource.testError1(new RemoteIsamSource("localhost", 1234));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
    @Test
    public void testError2() {
        try {
            TestCommonIsamSource.testError2(new RemoteIsamSource("localhost", 1234));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testUpdate() {
        try {
            TestCommonIsamSource.testUpdate(new RemoteIsamSource("localhost", 1234));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testDelete() {
        try {
            TestCommonIsamSource.testDelete(new RemoteIsamSource("localhost", 1234));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testLogging() {
        try {
            TestCommonIsamSource.testLogging(new RemoteIsamSource("localhost", 1234));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testConcurrency() {
        try {
            TestCommonIsamSource.testConcurrency(new RemoteIsamSource("localhost", 1234),
                                                 new RemoteIsamSource("localhost", 1234));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testConcurrencyHard() {
        try {
            TestCommonIsamSource.testConcurrencyHard(new RemoteIsamSource("localhost", 1234),
                                                     new RemoteIsamSource("localhost", 1234));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
}
